package gert {
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Graphics;
	import flash.display.PixelSnapping;
	import flash.display.Shape;
	import flash.display.Stage;
	import flash.utils.getTimer;
	
	/**
	 * ...
	 * @author Jesse Ringrose
	 */
	public class Game {
		
		public static const COLLISION_STATIC:int = 		1 << 0;
		public static const COLLISION_PLAYER:int = 		1 << 1;
		public static const COLLISION_ENTITIES:int = 	1 << 2;
		public static const COLLISION_SOUNDWAVES:int = 	1 << 3;
		
		public static const LEVELS:Array = [
		new Level1(),
		new Level2(),
		new Level3(),
		new Level4(),
		new Level4_2(),
		//new Level5(),
		new Level5_2(),
		new Level6(),
		new Level7(),
		new Level8(),
		new Level8_2(),
		new Level9(),
		new Level10(),
		new Level11(),
		];
		
		public static var DELTA_MILLIS:Number = 0;
		
		private var lastTime:Number = 0;
		
		public var currentLevelIndex:int = 4;
		
		public var level:Level;
		
		public var stage:Stage;
		
		public var canvas:BitmapData;
		public var canvasBitmap:Bitmap;
		private var blackAlphaRect:Shape;
		public var running:Boolean;
		
		public function Game(stage:Stage) {
			this.stage = stage;
			
			canvas = new BitmapData(stage.stageWidth, stage.stageHeight, false, 0x0);
			canvasBitmap = new Bitmap(canvas, PixelSnapping.NEVER, true);
			stage.addChild(canvasBitmap);
			
			blackAlphaRect = getColorRect(0x0, 0.1);
			
		}
		
		public function startLevel():void {
			lastTime = getTimer();
			
			if (level) {
				destroyLevel();
			}
			
			level = new Level(this, LEVELS[currentLevelIndex], stage);
			level.init();
			
			if(level.overlayClip){
				stage.addChild(level.overlayClip);
				level.overlayClip.gotoAndPlay(1);
			}
			
			running = true;
		}
		
		private function getColorRect(color:int, alpha:Number = 1):Shape {
			var rect:Shape = new Shape();
			var g:Graphics = rect.graphics;
			g.beginFill(color, alpha);
			g.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			g.endFill();
			return rect;
		}
		
		public function update():void {
			if (!running) return;
			
			DELTA_MILLIS = getTimer() - lastTime;
			lastTime = getTimer();
			
			canvas.lock();
			
			// fade out previously drawn things.
			if(!level.won)
				canvas.draw(blackAlphaRect);
			
			if(level)
				level.update(DELTA_MILLIS);
				
			canvas.unlock();
		}
		
		public function advanceLevel():void {
			canvas.draw(getColorRect(0xFFFFFF));
			currentLevelIndex++;
			if (currentLevelIndex >= LEVELS.length) {
				winGame();
			} else {
				startLevel();
			}
		}
		
		private function destroyLevel():void {
			if(level.overlayClip)
				level.overlayClip.parent.removeChild(level.overlayClip);
			level.dispose();
			level = null;
		}
		
		private function winGame():void {
			destroyLevel();
			running = false;
			stage.addChild(new WinGame());
		}
		
		public function loseGame():void {
			canvas.draw(getColorRect(0xFF0000));
			startLevel();
		}
		
	}

}