package gert {
	import Box2D.Collision.b2ContactID;
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2ContactListener;
	import Box2D.Dynamics.b2World;
	import Box2D.Dynamics.Contacts.b2Contact;
	import com.adobe.utils.ArrayUtil;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.Graphics;
	import flash.display.MovieClip;
	import flash.display.PixelSnapping;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.filters.BlurFilter;
	import flash.filters.DropShadowFilter;
	import flash.filters.GlowFilter;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.utils.setTimeout;
	import General.FpsCounter;
	import jrlib.util.Box2DUtil;
	import jrlib.util.KeyboardUtil;
	import jrlib.util.MathUtil;
	
	/**
	 * ...
	 * @author Jesse Ringrose
	 */
	public class Level {
		
		private var fpsMeter:FpsCounter = new FpsCounter();
		private var inited:Boolean = false;
		
		public var canvas:BitmapData;
		public var canvasBitmap:Bitmap;
		
		public var lineSprite:Sprite;
		public var lineGraphics:Graphics;
		
		public var m_world:b2World;
		private var levelClip:MovieClip;
		private var container:DisplayObjectContainer;
		public var player:Player;
		public var exit:ExitPoint;
		private var removeWaveList:Array = [];
		public var game:Game;
		
		public var overlayClip:MovieClip;
		private var contactListener:ContactListener;
		
		public var won:Boolean = false;
		public var lost:Boolean = false;
		
		public var soundWaves:Array = [];
		public var enemies:Array = [];
		public var killzones:Array = [];
		
		// bodies with displayObject userdata attached
		protected var trackedBodies:Array = [];
		
		public var debugSprite:Sprite;
		
		public function Level(game:Game, levelClip:MovieClip, container:DisplayObjectContainer) {
			this.game = game;
			this.levelClip = levelClip;
			this.container = container;
			this.canvas = game.canvas;
			this.canvasBitmap = game.canvasBitmap;
			
			if (Main.ENABLE_DEBUG) {
				debugSprite = new Sprite();
			}
		}
		
		public function init():void {
			m_world = Box2DUtil.createWorld(0, 0, true, debugSprite);
			
			contactListener = new ContactListener(this);
			m_world.SetContactListener(contactListener);
			
			lineSprite = new Sprite();
			lineGraphics = lineSprite.graphics;
			
			createCollisionBounds();
			
			createLevel();
			
			update(0);
			
			if(debugSprite)
				debugSprite.addChild(fpsMeter);
			
			if (debugSprite){
				debugSprite.visible = false;
				container.addChild(debugSprite);
				KeyboardUtil.inst.listenForKeyDown(KeyboardUtil.KEY_D, function():void {
					if(debugSprite)
						debugSprite.visible = !debugSprite.visible;
				});
			}
		}
		
		protected function createCollisionBounds():void {
			var PADDING:Number = 200;
			var WIDTH:Number = 800;
			var HEIGHT:Number = 600;
			Box2DUtil.createBox(m_world, WIDTH+PADDING*2, PADDING, -PADDING, -PADDING, false, false); // top
			Box2DUtil.createBox(m_world, WIDTH+PADDING*2, PADDING, -PADDING, HEIGHT, false, false); // bottom
			Box2DUtil.createBox(m_world, PADDING, HEIGHT+PADDING*2, -PADDING, -PADDING, false, false); // left
			Box2DUtil.createBox(m_world, PADDING, HEIGHT+PADDING*2, WIDTH, -PADDING, false, false); // right
		}
		
		private function createLevel():void {
			
			// create code-lines.
			if (levelClip.vertices) {
				for each(var arr:Array in levelClip.vertices) {
					Box2DUtil.createLines(m_world, [new Point(arr[0], arr[1]), new Point(arr[2], arr[3])]);
				}
			}
			
			// go through EVERY line and create it in the world !
			for (var i:int = 0; i < levelClip.numChildren; i++ ) {
				var child:DisplayObject = levelClip.getChildAt(i);
				var pos:Point = new Point(child.x, child.y);
				if(child is LinePlacement){
				
					// determine start/end pos.
					var startPos:Point = pos;
					var endPos:Point = new Point(
						child.x + Math.cos(child.rotation * MathUtil.DEG_TO_RAD) * child.scaleX * 100, 
						child.y + Math.sin(child.rotation * MathUtil.DEG_TO_RAD) * child.scaleX * 100
					);
					
					Box2DUtil.createLines(m_world, [startPos, endPos]);
				} else if (child is PlayerPlacement) {
					player = new Player(game, pos);
				} else if (child is ExitZonePlacement) {
					exit = new ExitPoint(game, child.getBounds(child.parent));
				} else if (child is EnemyPlacement) {
					var nme:Enemy = new Enemy(game, pos);
					enemies.push(nme);
				} else if (child is KillZonePlacement) {
					var killzone:KillZone = new KillZone(game, child.getBounds(child.parent));
					killzones.push(killzone);
				}
			}
			
			if (Object(levelClip).hasOwnProperty("overlay")) {
				overlayClip = Object(levelClip).overlay;
			}
		}
		
		public function update(deltaT:Number):void {
			lineGraphics.clear();
			lineGraphics.lineStyle(1, 0xFFFFFF, 1);
			
			player.update(deltaT);
			
			for each(var wave:SoundWave in soundWaves) {
				wave.update(deltaT);
				
				if (wave.lifetime <= 0)
					removeWaveList.push(wave);
			}
			
			for each(wave in removeWaveList) {
				ArrayUtil.removeValueFromArray(soundWaves, wave);
			}
			removeWaveList.length = 0;
			
			for each(var nme:Enemy in enemies) {
				nme.update(deltaT);
			}
			
			exit.update(deltaT);
			
			Box2DUtil.updateWorld(m_world, 50, 5);
			
			for each(var body:b2Body in trackedBodies) {
				var nametag:Sprite = body.GetUserData();
				
				Box2DUtil.updateDisplayObject(nametag, body);
			}
			
			fpsMeter.update();
			
			// draw layers.
			canvas.draw(lineSprite);
			
			// remove all children
			while (lineSprite.numChildren > 0)
				lineSprite.removeChildAt(0);
			
			// call postupdates !
			exit.postupdate();
			
			for each(nme in enemies) {
				nme.postupdate();
			}
			
			/*if (overlayClip && Math.random() < 0.1) {
				var m:Matrix = new Matrix();
				m.tx = Math.random() * 4 - 2;
				m.ty = Math.random() * 4 - 2;
				canvas.draw(overlayClip, m);
			}*/
			
			if (won) win();
			if (lost) lose();
		}
		
		public function makeSound(waveCount:int, startPos:b2Vec2, lifetimeSeconds:Number = 1, fromEnemy:Boolean = false, randomLifetime:Number = 1):void {
			var speed:Number = 1 * 2;
			lifetimeSeconds *= 0.5;
			var angleOffset:Number = Math.random() * Math.PI * 2;
			for (var i:int = 0; i < waveCount; i++ ) {
				//var angleRandom:Number = (Math.random() * 2 - 1) * Math.PI * 2 * (1 / waveCount) * 0.5;
				var angleRandom:Number = 0;
				var angle:Number = Math.PI * 2 * (i / waveCount) + angleOffset + angleRandom;
				var wave:SoundWave = new SoundWave(game);
				wave.fromEnemy = fromEnemy;
				wave.create(angle, startPos.x, startPos.y, speed, lifetimeSeconds + Math.random()*randomLifetime);
				
				soundWaves.push(wave);
			}
		}
		
		public function win():void {
			// tadaa !!!
			//game.advanceLevel();
			if (!player.body.IsActive()) return;
			player.body.SetActive(false);
			
			// die. restart after a while i guess.
			makeSound(50, player.body.GetPosition(), 100, false);
			setTimeout(function():void {
				game.advanceLevel();
			}, 3300);
			
			new SNDCompleteLevel().play();
		}
		
		public function lose():void {
			if (!player.body.IsActive()) return;
			player.body.SetActive(false);
			
			// die. restart after a while i guess.
			setTimeout(function():void {
				game.loseGame();
			}, 5000);
			
			setTimeout(function():void {
				(new SNDDoorClose()).play();
			}, 4000);
			//game.startLevel();
		}
		
		public function dispose():void {
			player.dispose();
			player = null;
			
			if(debugSprite){
				debugSprite.parent.removeChild(debugSprite);
				debugSprite = null;
			}
			m_world = null;
		}
		
	}

}
