package gert {
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2FilterData;
	import Box2D.Dynamics.b2World;
	import flash.geom.Point;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundTransform;
	import flash.utils.setTimeout;
	import jrlib.math.Vec2D;
	import jrlib.util.Box2DUtil;
	import jrlib.util.KeyboardUtil;
	import jrlib.util.MathUtil;
	
	/**
	 * ...
	 * @author Jesse Ringrose
	 */
	public class Player {
		
		private static const WALK_SPEED:Number = 3;
		private static const SNEAK_SPEED:Number = 1.2;
		
		private static const MS_BETWEEN_FOOTSTEPS:Number = 500;
		
		// tap parameters are all scaled by how long spacebar is held down for.
		private static const MAX_TAP_POWER:Number = 1000;	// how many milliseconds to reach full tap power.
		private static const MIN_TAP_POWER:Number = 100;	// minimum milliseconds the tap length can be.
		private static const MAX_TAP_WAVES:int = 60;		// how many waves are sent out on a full-power tap.
		private static const MAX_TAP_LIFETIME:Number = 3.0;	// seconds until tap waves begin to disappear.
		
		private var game:Game;
		private var tapPower:Number = 0;
		
		private var lastStepX:Number = 0;
		private var lastStepY:Number = 0;
		
		private var footstepInc:int =  Math.random() * 2;
		private var footstepCount:Number = MS_BETWEEN_FOOTSTEPS - 1;
		
		private var nmeSound:SoundChannel;
		private var targetVolume:Number = 0;
		
		public var body:b2Body;
		
		public function Player(game:Game, startPoint:Point) {
			this.game = game;
			body = Box2DUtil.createOrientedBox(game.level.m_world, 7, 7, startPoint.x, startPoint.y, true, true, 0.1, 0.5, 0);
			body.SetUserData(this);
			body.SetFixedRotation(true);
			
			var filter:b2FilterData = new b2FilterData();
			filter.categoryBits = Game.COLLISION_PLAYER;
			body.GetFixtureList().SetFilterData(filter);
			
			nmeSound = new SNDMonster().play(0, int.MAX_VALUE, new SoundTransform(0, 0));
		}
		
		public function update(deltaT:Number):void {
			if (!body.IsActive()) return;
			
			var playerPos:b2Vec2 = Box2DUtil.getBodyPosition(body);
			if (game.level.exit.isInExit(playerPos.x, playerPos.y)) {
				body.SetLinearVelocity(new b2Vec2(0, 0));
				// level is done !!!
				game.level.won = true;
				return;
			}
			
			for each(var kz:KillZone in game.level.killzones) {
				if (kz.rect.contains(playerPos.x, playerPos.y)) {
					new SNDTrap().play();
					die();
					return;
				}
			}
			
			var closestEnemy:Enemy = null;
			var closestDist:Number = Number.MAX_VALUE;
			targetVolume = 0;
			for each(var nme:Enemy in game.level.enemies) {
				if(nme.isEngaged()){
					var dist:Number = MathUtil.distanceBetween(nme.pos.x, nme.pos.y, playerPos.x, playerPos.y);
					if (dist < closestDist) {
						closestEnemy = nme;
						closestDist = dist;
					}
				}
				if (nme.isInRect(playerPos.x, playerPos.y)) {
					die();
					return;
				}
			}
			
			// scale sound if nme is near !
			if (closestEnemy) {
				targetVolume = Math.max(0.3, 1 - Math.min(1, closestDist / 200)) * 2;
			}
			var st:SoundTransform = nmeSound.soundTransform;
			st.volume = st.volume + (targetVolume - st.volume) * 0.1;
			nmeSound.soundTransform = st;
			
			var vx:Number = 0;
			var vy:Number = 0;
			if (KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_LEFT)) {
				vx -= 1;
			}
			if (KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_RIGHT)) {
				vx += 1;
			}
			if (KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_UP)) {
				vy -= 1;
			}
			if (KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_DOWN)) {
				vy += 1;
			}
			
			if (KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_SPACE)) {
				tapPower += deltaT;
			} else {
				if (tapPower > 0) {
					tap(tapPower);
					tapPower = 0;
				}
			}
			
			var isSneaking:Boolean = KeyboardUtil.inst.isKeyDown(KeyboardUtil.KEY_SHIFT);
			
			if (vx != 0 || vy != 0) {
				if(isSneaking){
					vx *= SNEAK_SPEED;
					vy *= SNEAK_SPEED;
				} else {
					footstepCount += deltaT;
					vx *= WALK_SPEED;
					vy *= WALK_SPEED;
				}
				if (footstepCount > MS_BETWEEN_FOOTSTEPS) {
					footstepCount = 0;
					makeFootstep();
				}
				
				body.SetLinearVelocity(new b2Vec2(vx, vy));
				body.SetAwake(true);
			} else {
				body.SetLinearVelocity(new b2Vec2(0, 0));
				//footstepCount = MS_BETWEEN_FOOTSTEPS*1.1;
			}
		}
		
		public function die():void {
			(new SNDDie()).play();
			
			game.level.makeSound(100, body.GetPosition(), 10, true);
			game.level.lost = true;
		}
		
		public function tap(power:Number):void {
			var pos:b2Vec2 = body.GetPosition();
			
			power = Math.min(MAX_TAP_POWER, Math.max(MIN_TAP_POWER, power));
			power = power /= MAX_TAP_POWER;
			
			// spew out a sound !
			game.level.makeSound(MAX_TAP_WAVES * power, pos, power * MAX_TAP_LIFETIME, false, power * 1 + 0.2);
			
			var snd:Sound = new SNDTap();
			snd.play(0, 0, new SoundTransform(power * 3));
		}
		
		private static const LEFT_FOOTSTEPS:Array = [SNDFootstepA];
		private static const RIGHT_FOOTSTEPS:Array = [SNDFootstepB];
		
		public function makeFootstep():void {
			footstepInc++;
			
			var pos:b2Vec2 = body.GetPosition().Copy();
			
			
			// offset the step pos a bit.
			var ang:Number = Math.atan2(pos.y - lastStepY, pos.x - lastStepX);
			ang += (footstepInc % 2 == 0) ? Math.PI / -2 : Math.PI / 2;
			
			if(lastStepX != 0 && lastStepY != 0){
				pos.x += Math.cos(ang) * 0.06;
				pos.y += Math.sin(ang) * 0.06;
			}
			
			lastStepX = body.GetPosition().x;
			lastStepY = body.GetPosition().y;
			
			var choices:Array = footstepInc % 2 == 0 ? LEFT_FOOTSTEPS : RIGHT_FOOTSTEPS;
			(new (choices[int(choices.length * Math.random())])()).play();
			
			// spew out a sound !
			game.level.makeSound(20, pos, 0.5);
		}
		
		public function dispose():void {
			nmeSound.stop();
			nmeSound = null;
		}
		
	}

}