package gert {
	import Box2D.Common.Math.b2Vec2;
	import Box2D.Dynamics.b2Body;
	import Box2D.Dynamics.b2FilterData;
	import flash.display.Graphics;
	import flash.display.LineScaleMode;
	import flash.display.Sprite;
	import jrlib.util.Box2DUtil;
	import jrlib.util.MathUtil;
	
	/**
	 * ...
	 * @author Jesse Ringrose
	 */
	public class SoundWave {
		private var game:Game;
		
		public var body:b2Body;
		
		public var lastX:Number = 0;
		public var lastY:Number = 0;
		
		public var startLifetime:Number = 0; 	// seconds.
		public var lifetime:Number = 0;			// seconds.
		
		public var startX:Number = 0;
		public var startY:Number = 0;
		
		public var numFrames:int = 0 + Math.random() * 2;
		
		public var fromEnemy:Boolean = false;
		
		public function SoundWave(game:Game) {
			this.game = game;
		}
		
		public function create(angle:Number, startX:Number, startY:Number, speed:Number, lifetime:Number = 3):void {
			this.startX = startX;
			this.startY = startY;
			
			body = Box2DUtil.createBox(game.level.m_world, 2, 2, startX * Box2DUtil.SCALING_FACTOR, startY * Box2DUtil.SCALING_FACTOR, true, true, 3, 0, 1);
			body.SetUserData(this);
			body.SetBullet(true);
			var filter:b2FilterData = new b2FilterData();
			filter.categoryBits = Game.COLLISION_SOUNDWAVES;
			filter.maskBits = Game.COLLISION_STATIC | Game.COLLISION_ENTITIES;
			body.GetFixtureList().SetFilterData(filter);
			
			body.SetFixedRotation(true);
			
			body.ApplyImpulse(new b2Vec2(
				Math.cos(angle) * speed * 0.1,
				Math.sin(angle) * speed * 0.1
			), body.GetPosition());
			
			this.lifetime = lifetime;
			this.startLifetime = lifetime;
		}
		
		public function update(deltaT:Number):void {
			lifetime -= deltaT / 1000;
			
			if (lifetime <= 0) {
				// remove !
				dispose();
				return;
			}
			
			// bug i guess.
			if (!body) return;
			
			numFrames++;
			
			// draw single dots !
			
			// TODO: draw line from last pos to current pos ??
			var pos:b2Vec2 = body.GetPosition();
			pos.x *= Box2DUtil.SCALING_FACTOR;
			pos.y *= Box2DUtil.SCALING_FACTOR;
			/*game.level.canvas.setPixel(pos.x, pos.y, 0xFFFFFF);*/
			var g:Graphics = game.level.lineGraphics;
			
			
			if (lastX != 0 && lastY != 0) {
				var color:uint = 0xFFFFFF;
				var alpha:Number = lifetime / startLifetime * 0.6;
				if(!fromEnemy){
					for each(var nme:Enemy in game.level.enemies) {
						if (nme.isInRect(pos.x, pos.y)) {
							color = 0xFF0000;
							alpha = 1;
							nme.targetX = startX * Box2DUtil.SCALING_FACTOR;
							nme.targetY = startY * Box2DUtil.SCALING_FACTOR;
							
							// draw circle thingy.
							/*g.lineStyle();
							g.beginFill(0xFF0000, 1);
							g.drawCircle(pos.x, pos.y, 3);
							g.endFill();*/
							
							dispose();
							break;
						}
					}
					
					if (game.level.exit.isInExit(pos.x, pos.y)){
						color = 0xFFFFFF;
						alpha = 1;
						lifetime += deltaT / 1000;
					}
					
					for each(var zone:KillZone in game.level.killzones) {
						if (zone.rect.contains(pos.x, pos.y)) {
							color = 0xFF0000;
							alpha = 0.5;
							//lifetime += deltaT / 1000;
							break;
						}
					}
				} else {
					color = 0xFF0000;
				}
				
				//if(numFrames%2 == 0){
					g.lineStyle(1, color, alpha);
					g.moveTo(lastX, lastY);
					g.lineTo(pos.x, pos.y);
				//}
				
				// draw curvey thing
				/*var curveThing:Sprite = new SoundCurve();
				curveThing.x = pos.x;
				curveThing.y = pos.y;
				curveThing.scaleX = 
				curveThing.scaleY = alpha * 0.5 + 0.5;
				curveThing.rotation = Math.atan2(pos.y-lastY, pos.x-lastX) * MathUtil.RAD_TO_DEG + 90;
				game.level.lineSprite.addChild(curveThing);*/
			}
			
			lastX = pos.x;
			lastY = pos.y;
		}
		
		public function dispose():void {
			game.level.m_world.DestroyBody(body);
			body = null;
			lifetime = -1000;
		}
		
	}

}